/* libMath: Mathematical library

 Bugs can be reported to yokochi@pharm.hokudai.ac.jp

 This work is a part of Olivia development project.
   http://fermi.pharm.hokudai.ac.jp/olivia/
*/

#include "libMath.h"

/* C99 <math.h> extensions */

#ifndef HAVE_C99MATHEXT

#ifndef __FLT_MIN__
#define __FLT_MIN__ 1.17549435e-38F
#endif

#ifndef __FLT_MAX__
#define __FLT_MAX__ 3.40282347e+38F
#endif

#define MAKE_INFF(s) (__FLT_MAX__*__FLT_MAX__*(s))

int __isnormalf(float x)
{

#ifndef __sun__
 float __fabsf = fabsf(x);
#else
 float __fabsf = fabs(x);
#endif

 if (x != x)
  return 0;

 return __fabsf < MAKE_INFF(1) && __fabsf >= __FLT_MIN__;
}

#ifndef __DBL_MIN__
#define __DBL_MIN__ 2.2250738585072014e-308
#endif

#ifndef __DBL_MAX__
#define __DBL_MAX__ 1.7976931348623157e+308
#endif

#define MAKE_INF(s) (__DBL_MAX__*__DBL_MAX__*(s))

int __isnormal(double x)
{
 double __fabs = fabs(x);

 if (x != x)
  return 0;

 return __fabs < MAKE_INF(1) && __fabs >= __DBL_MIN__;
}

int __isfinitef(float x)
{

#ifndef __sun__
 float __fabsf = fabsf(x);
#else
 float __fabsf = fabs(x);
#endif

 return x == x && __fabsf != MAKE_INFF(1);
}

int __isfinite(double x)
{
 double __fabs = fabs(x);

 return x == x && __fabs != MAKE_INF(1);
}

#endif

float __powif(float x, int n)
{
 float y;

 if (!isfinite(x))
  goto escape;

 switch (n) {
 case 2:
  return x * x;
  break;

 case 1:
  return x;
  break;

 case 0:
  return 1.0;
  break;

 case -1:
  return 1.0 / x;
  break;

 case -2:
  return 1.0 / x / x;
  break;

 case 6:
 case 3:
 case -3:
 case -6:
  y = x * x * x;

  switch (n) {
  case 6:
   return y * y;
   break;

  case 3:
   return y;
   break;

  case -3:
   return 1.0 / y;
   break;

  case -6:
   return 1.0 / y / y;
   break;
  }
  break;
 }

escape:
#ifndef __sun__
 return powf(x, (float) (n));
#else
 return pow(x, (double) (n));
#endif
}

double __powi(double x, int n)
{
 double y;

 if (!isfinite(x))
  goto escape;

 switch (n) {
 case 2:
  return x * x;
  break;

 case 1:
  return x;
  break;

 case 0:
  return 1.0;
  break;

 case -1:
  return 1.0 / x;
  break;

 case -2:
  return 1.0 / x / x;
  break;

 case 6:
 case 3:
 case -3:
 case -6:
  y = x * x * x;

  switch (n) {
  case 6:
   return y * y;
   break;

  case 3:
   return y;
   break;

  case -3:
   return 1.0 / y;
   break;

  case -6:
   return 1.0 / y / y;
   break;
  }
  break;
 }

escape:return pow(x, (double) (n));
}

/* complex */

complex Cadd(complex a, complex b)
{
 complex c;

 c.r = a.r + b.r;
 c.i = a.i + b.i;

 return c;
}

complex Csub(complex a, complex b)
{
 complex c;

 c.r = a.r - b.r;
 c.i = a.i - b.i;

 return c;
}

complex Cmul(complex a, complex b)
{
 complex c;

 c.r = a.r * b.r - a.i * b.i;
 c.i = a.i * b.r + a.r * b.i;

 return c;
}

complex Conj(complex z)
{
 complex c;

 c.r = z.r;
 c.i = -z.i;

 return c;
}

complex Cdiv(complex a, complex b)
{
 complex c;
 float r, den;

 if (fabs(b.r) >= fabs(b.i)) {
  r = b.i / b.r;
  den = b.r + r * b.i;
  c.r = (a.r + r * a.i) / den;
  c.i = (a.i - r * a.r) / den;
 }

 else {
  r = b.r / b.i;
  den = b.i + r * b.r;
  c.r = (a.r * r + a.i) / den;
  c.i = (a.i * r - a.r) / den;
 }

 return c;
}

float Cabs(complex z)
{
 float x, y, r, ans;

 x = fabs(z.r);
 y = fabs(z.i);

 if (x == 0.0)
  ans = y;

 else if (y == 0.0)
  ans = x;

 else if (x > y) {
  r = y / x;
  ans = x * sqrt(1.0 + r * r);
 }

 else {
  r = x / y;
  ans = y * sqrt(1.0 + r * r);
 }

 return ans;
}

complex Csqrt(complex z)
{
 complex c;
 float x, y, w, r;

 if (z.r == 0.0 && z.i == 0.0) {
  c.r = 0.0;
  c.i = 0.0;
 }

 else {
  x = fabs(z.r);
  y = fabs(z.i);

  if (x >= y) {
   r = y / x;
   w = sqrt(x) * sqrt(0.5 * (1.0 + sqrt(1.0 + r * r)));
  }

  else {
   r = x / y;
   w = sqrt(y) * sqrt(0.5 * (r + sqrt(1.0 + r * r)));
  }

  if (z.r >= 0.0) {
   c.r = w;
   c.i = z.i / (2.0 * w);
  }

  else {
   c.i = (z.i >= 0.0) ? w : -w;
   c.r = z.i / (2.0 * c.i);
  }
 }

 return c;
}

complex RCmul(float x, complex a)
{
 complex c;

 c.r = x * a.r;
 c.i = x * a.i;

 return c;
}

complex Cexp(complex z)
{
 complex c;
 float r;

 r = exp(z.r);

 c.r = r * cos(z.i);
 c.i = r * sin(z.i);

 return c;
}

/* double complex */

doublecomplex DCadd(doublecomplex a, doublecomplex b)
{
 doublecomplex c;

 c.r = a.r + b.r;
 c.i = a.i + b.i;

 return c;
}

doublecomplex DCsub(doublecomplex a, doublecomplex b)
{
 doublecomplex c;

 c.r = a.r - b.r;
 c.i = a.i - b.i;

 return c;
}

doublecomplex DCmul(doublecomplex a, doublecomplex b)
{
 doublecomplex c;

 c.r = a.r * b.r - a.i * b.i;
 c.i = a.i * b.r + a.r * b.i;

 return c;
}

doublecomplex DConj(doublecomplex z)
{
 doublecomplex c;

 c.r = z.r;
 c.i = -z.i;

 return c;
}

doublecomplex DCdiv(doublecomplex a, doublecomplex b)
{
 doublecomplex c;
 double r, den;

 if (fabs(b.r) >= fabs(b.i)) {
  r = b.i / b.r;
  den = b.r + r * b.i;
  c.r = (a.r + r * a.i) / den;
  c.i = (a.i - r * a.r) / den;
 }

 else {
  r = b.r / b.i;
  den = b.i + r * b.r;
  c.r = (a.r * r + a.i) / den;
  c.i = (a.i * r - a.r) / den;
 }

 return c;
}

double DCabs(doublecomplex z)
{
 double x, y, r, ans;

 x = fabs(z.r);
 y = fabs(z.i);

 if (x == 0.0)
  ans = y;

 else if (y == 0.0)
  ans = x;

 else if (x > y) {
  r = y / x;
  ans = x * sqrt(1.0 + r * r);
 }

 else {
  r = x / y;
  ans = y * sqrt(1.0 + r * r);
 }

 return ans;
}

doublecomplex DCsqrt(doublecomplex z)
{
 doublecomplex c;
 double x, y, w, r;

 if (z.r == 0.0 && z.i == 0.0) {
  c.r = 0.0;
  c.i = 0.0;
 }

 else {
  x = fabs(z.r);
  y = fabs(z.i);

  if (x >= y) {
   r = y / x;
   w = sqrt(x) * sqrt(0.5 * (1.0 + sqrt(1.0 + r * r)));
  }

  else {
   r = x / y;
   w = sqrt(y) * sqrt(0.5 * (r + sqrt(1.0 + r * r)));
  }

  if (z.r >= 0.0) {
   c.r = w;
   c.i = z.i / (2.0 * w);
  }

  else {
   c.i = (z.i >= 0.0) ? w : -w;
   c.r = z.i / (2.0 * c.i);
  }
 }

 return c;
}

doublecomplex RDCmul(double x, doublecomplex a)
{
 doublecomplex c;

 c.r = x * a.r;
 c.i = x * a.i;

 return c;
}

doublecomplex DCexp(doublecomplex z)
{
 doublecomplex c;
 double r;

 r = exp(z.r);

 c.r = r * cos(z.i);
 c.i = r * sin(z.i);

 return c;
}

/* hyper complex */

hypercomplex HCadd(hypercomplex a, hypercomplex b)
{
 hypercomplex c;

 c.x = Cadd(a.x, b.x);
 c.y = Cadd(a.y, b.y);

 return c;
}

hypercomplex HCsub(hypercomplex a, hypercomplex b)
{
 hypercomplex c;

 c.x = Csub(a.x, b.x);
 c.y = Csub(a.y, b.y);

 return c;
}

hypercomplex HCmul(hypercomplex a, hypercomplex b)
{
 hypercomplex c;

 c.x = Cmul(a.x, b.x);
 c.y = Cmul(a.y, b.y);

 return c;
}

hypercomplex HConj(hypercomplex z)
{
 hypercomplex c;

 c.x = Conj(z.x);
 c.y = Conj(z.y);

 return c;
}

hypercomplex HCtp(hypercomplex z)
{
 hypercomplex c;

 c.x = z.y;
 c.y = z.x;

 return c;
}
